@@ -61,19 +61,21 @@ typedef struct {
 }
 	NadpReaderData;
 
-static GList            *get_list_of_desktop_paths( const NadpDesktopProvider *provider, GSList **mesages );
+static GList            *get_list_of_desktop_paths( NadpDesktopProvider *provider, GSList **mesages );
 static void              get_list_of_desktop_files( const NadpDesktopProvider *provider, GList **files, const gchar *dir, GSList **messages );
 static gboolean          is_already_loaded( const NadpDesktopProvider *provider, GList *files, const gchar *desktop_id );
 static GList            *desktop_path_from_id( const NadpDesktopProvider *provider, GList *files, const gchar *dir, const gchar *id );
 static NAIFactoryObject *item_from_desktop_path( const NadpDesktopProvider *provider, DesktopPath *dps, GSList **messages );
+static NAIFactoryObject *item_from_desktop_file( const NadpDesktopProvider *provider, NadpDesktopFile *ndf, GSList **messages );
 static void              desktop_weak_notify( NadpDesktopFile *ndf, GObject *item );
 static void              free_desktop_paths( GList *paths );
 
-static gboolean          read_done_desktop_is_writable( const NAIFactoryProvider *provider, NAObjectItem *item, NadpReaderData *reader_data, GSList **messages );
-static void              read_subitems_key( const NAIFactoryProvider *provider, NAObjectItem *item, NadpReaderData *reader_data, GSList **messages );
-static void              read_done_load_profiles( const NAIFactoryProvider *provider, NAObjectAction *action, NadpReaderData *data, GSList **messages );
+static void              read_start_read_subitems_key( const NAIFactoryProvider *provider, NAObjectItem *item, NadpReaderData *reader_data, GSList **messages );
+static void              read_start_profile_attach_profile( const NAIFactoryProvider *provider, NAObjectProfile *profile, NadpReaderData *reader_data, GSList **messages );
+
+static gboolean          read_done_item_is_writable( const NAIFactoryProvider *provider, NAObjectItem *item, NadpReaderData *reader_data, GSList **messages );
+static void              read_done_action_read_profiles( const NAIFactoryProvider *provider, NAObjectAction *action, NadpReaderData *data, GSList **messages );
 static void              read_done_action_load_profile( const NAIFactoryProvider *provider, NadpReaderData *reader_data, const gchar *profile_id, GSList **messages );
-static void              read_done_attach_profile( const NAIFactoryProvider *provider, NAObjectProfile *profile, NadpReaderData *reader_data, GSList **messages );
 
 /*
  * Returns an unordered list of NAIFactoryObject-derived objects
@@ -94,6 +96,7 @@ nadp_iio_provider_read_items( const NAIIOProvider *provider, GSList **messages )
 	g_return_val_if_fail( NA_IS_IIO_PROVIDER( provider ), NULL );
 
 	items = NULL;
+	nadp_desktop_provider_release_monitors( NADP_DESKTOP_PROVIDER( provider ));
 
 	desktop_paths = get_list_of_desktop_paths( NADP_DESKTOP_PROVIDER( provider ), messages );
 	for( ip = desktop_paths ; ip ; ip = ip->next ){
@@ -122,7 +125,7 @@ nadp_iio_provider_read_items( const NAIIOProvider *provider, GSList **messages )
  * the ordered of preference (most preferred first)
  */
 static GList *
-get_list_of_desktop_paths( const NadpDesktopProvider *provider, GSList **messages )
+get_list_of_desktop_paths( NadpDesktopProvider *provider, GSList **messages )
 {
 	GList *files;
 	GSList *xdg_dirs, *idir;
@@ -137,11 +140,12 @@ get_list_of_desktop_paths( const NadpDesktopProvider *provider, GSList **message
 	 */
 	for( idir = xdg_dirs ; idir ; idir = idir->next ){
 
-		/* explore chaque N-A candidate subdirectory for each XDG dir
+		/* explore each N-A candidate subdirectory for each XDG dir
 		 */
 		for( isub = subdirs ; isub ; isub = isub->next ){
 
 			dir = g_build_filename(( gchar * ) idir->data, ( gchar * ) isub->data, NULL );
+			nadp_desktop_provider_add_monitor( provider, dir );
 			get_list_of_desktop_files( provider, &files, dir, messages );
 			g_free( dir );
 		}
@@ -248,18 +252,29 @@ desktop_path_from_id( const NadpDesktopProvider *provider, GList *files, const g
 static NAIFactoryObject *
 item_from_desktop_path( const NadpDesktopProvider *provider, DesktopPath *dps, GSList **messages )
 {
-	static const gchar *thisfn = "nadp_reader_item_from_desktop_path";
-	NAIFactoryObject *item;
 	NadpDesktopFile *ndf;
-	gchar *type;
-	NadpReaderData *reader_data;
-	gchar *id;
 
 	ndf = nadp_desktop_file_new_from_path( dps->path );
 	if( !ndf ){
 		return( NULL );
 	}
 
+	return( item_from_desktop_file( provider, ndf, messages ));
+}
+
+/*
+ * Returns a newly allocated NAIFactoryObject-derived object, initialized
+ * from the .desktop file
+ */
+static NAIFactoryObject *
+item_from_desktop_file( const NadpDesktopProvider *provider, NadpDesktopFile *ndf, GSList **messages )
+{
+	static const gchar *thisfn = "nadp_reader_item_from_desktop_file";
+	NAIFactoryObject *item;
+	gchar *type;
+	NadpReaderData *reader_data;
+	gchar *id;
+
 	item = NULL;
 	type = nadp_desktop_file_get_file_type( ndf );
 
@@ -320,6 +335,133 @@ free_desktop_paths( GList *paths )
 	g_list_free( paths );
 }
 
+/**
+ * nadp_reader_iimporter_import_from_uri:
+ * @instance: the #NAIImporter provider.
+ * @parms: a #NAIImporterUriParms structure.
+ *
+ * Imports an item.
+ *
+ * Returns: the import operation code.
+ *
+ * As soon as we have a valid .desktop file, we are most probably willing
+ * to successfully import an action or a menu of it.
+ *
+ * GLib does not have any primitive to load a key file from an uri.
+ * So we have to load the file into memory, and then try to load the key
+ * file from the memory data.
+ */
+guint
+nadp_reader_iimporter_import_from_uri( const NAIImporter *instance, NAIImporterImportFromUriParms *parms )
+{
+	static const gchar *thisfn = "nadp_reader_iimporter_import_from_uri";
+	guint code;
+	NadpDesktopFile *ndf;
+	NAIImporterManageImportModeParms manage_parms;
+
+	g_debug( "%s: instance=%p, parms=%p", thisfn, ( void * ) instance, ( void * ) parms );
+
+	g_return_val_if_fail( NA_IS_IIMPORTER( instance ), IMPORTER_CODE_PROGRAM_ERROR );
+	g_return_val_if_fail( NADP_IS_DESKTOP_PROVIDER( instance ), IMPORTER_CODE_PROGRAM_ERROR );
+
+	code = IMPORTER_CODE_NOT_WILLING_TO;
+
+	ndf = nadp_desktop_file_new_from_uri( parms->uri );
+	if( ndf ){
+		parms->exist = FALSE;
+		parms->import_mode = IMPORTER_MODE_NO_IMPORT;
+		parms->imported = ( NAObjectItem * ) item_from_desktop_file(
+				( const NadpDesktopProvider * ) NADP_DESKTOP_PROVIDER( instance ),
+				ndf, &parms->messages );
+
+		if( parms->imported ){
+			code = IMPORTER_CODE_OK;
+			g_return_val_if_fail( NA_IS_OBJECT_ITEM( parms->imported ), IMPORTER_CODE_NOT_WILLING_TO );
+
+			manage_parms.version = 1;
+			manage_parms.imported = parms->imported;
+			manage_parms.check_fn = parms->check_fn;
+			manage_parms.check_fn_data = parms->check_fn_data;
+			manage_parms.ask_fn = parms->ask_fn;
+			manage_parms.ask_fn_data = parms->ask_fn_data;
+			manage_parms.asked_mode = parms->asked_mode;
+			manage_parms.messages = parms->messages;
+
+			code = na_iimporter_manage_import_mode( &manage_parms );
+
+			parms->exist = manage_parms.exist;
+			parms->import_mode = manage_parms.import_mode;
+		}
+
+		if( code != IMPORTER_CODE_OK ){
+			if( parms->imported ){
+				g_object_unref( parms->imported );
+				parms->imported = NULL;
+			}
+		}
+	}
+
+	return( code );
+}
+
+/*
+ * at this time, the object has been allocated and its id has been set
+ * read here the subitems key, which may be 'Profiles' or 'ItemsList'
+ * depending of the exact class of the NAObjectItem
+ */
+void
+nadp_reader_ifactory_provider_read_start( const NAIFactoryProvider *reader, void *reader_data, const NAIFactoryObject *serializable, GSList **messages )
+{
+	static const gchar *thisfn = "nadp_reader_ifactory_provider_read_start";
+
+	g_return_if_fail( NA_IS_IFACTORY_PROVIDER( reader ));
+	g_return_if_fail( NADP_IS_DESKTOP_PROVIDER( reader ));
+	g_return_if_fail( NA_IS_IFACTORY_OBJECT( serializable ));
+
+	if( !NADP_DESKTOP_PROVIDER( reader )->private->dispose_has_run ){
+
+		g_debug( "%s: reader=%p (%s), reader_data=%p, serializable=%p (%s), messages=%p",
+				thisfn,
+				( void * ) reader, G_OBJECT_TYPE_NAME( reader ),
+				( void * ) reader_data,
+				( void * ) serializable, G_OBJECT_TYPE_NAME( serializable ),
+				( void * ) messages );
+
+		if( NA_IS_OBJECT_ITEM( serializable )){
+			read_start_read_subitems_key( reader, NA_OBJECT_ITEM( serializable ), ( NadpReaderData * ) reader_data, messages );
+		}
+
+		if( NA_IS_OBJECT_PROFILE( serializable )){
+			read_start_profile_attach_profile( reader, NA_OBJECT_PROFILE( serializable ), ( NadpReaderData * ) reader_data, messages );
+		}
+	}
+}
+
+static void
+read_start_read_subitems_key( const NAIFactoryProvider *provider, NAObjectItem *item, NadpReaderData *reader_data, GSList **messages )
+{
+	GSList *subitems;
+	gboolean key_found;
+
+	subitems = nadp_desktop_file_get_string_list( reader_data->ndf,
+			NADP_GROUP_DESKTOP,
+			NA_IS_OBJECT_ACTION( item ) ? NADP_KEY_PROFILES : NADP_KEY_ITEMS_LIST,
+			&key_found,
+			NULL );
+
+	if( key_found ){
+		na_object_set_items_slist( item, subitems );
+	}
+
+	na_core_utils_slist_free( subitems );
+}
+
+static void
+read_start_profile_attach_profile( const NAIFactoryProvider *provider, NAObjectProfile *profile, NadpReaderData *reader_data, GSList **messages )
+{
+	na_object_attach_profile( reader_data->action, profile );
+}
+
 /*
  * reading any data from a desktop file requires:
  * - a NadpDesktopFile object which has been initialized with the .desktop file
@@ -442,77 +584,54 @@ nadp_reader_ifactory_provider_read_done( const NAIFactoryProvider *reader, void
 	static const gchar *thisfn = "nadp_reader_ifactory_provider_read_done";
 	gboolean writable;
 
-	g_debug( "%s: reader=%p (%s), reader_data=%p, serializable=%p (%s), messages=%p",
-			thisfn,
-			( void * ) reader, G_OBJECT_TYPE_NAME( reader ),
-			( void * ) reader_data,
-			( void * ) serializable, G_OBJECT_TYPE_NAME( serializable ),
-			( void * ) messages );
-
 	g_return_if_fail( NA_IS_IFACTORY_PROVIDER( reader ));
 	g_return_if_fail( NADP_IS_DESKTOP_PROVIDER( reader ));
 	g_return_if_fail( NA_IS_IFACTORY_OBJECT( serializable ));
 
 	if( !NADP_DESKTOP_PROVIDER( reader )->private->dispose_has_run ){
 
+		g_debug( "%s: reader=%p (%s), reader_data=%p, serializable=%p (%s), messages=%p",
+				thisfn,
+				( void * ) reader, G_OBJECT_TYPE_NAME( reader ),
+				( void * ) reader_data,
+				( void * ) serializable, G_OBJECT_TYPE_NAME( serializable ),
+				( void * ) messages );
+
 		if( NA_IS_OBJECT_ITEM( serializable )){
-			writable = read_done_desktop_is_writable( reader, NA_OBJECT_ITEM( serializable ), ( NadpReaderData * ) reader_data, messages );
+			writable = read_done_item_is_writable( reader, NA_OBJECT_ITEM( serializable ), ( NadpReaderData * ) reader_data, messages );
 			na_object_set_readonly( serializable, !writable );
-			read_subitems_key( reader, NA_OBJECT_ITEM( serializable ), ( NadpReaderData * ) reader_data, messages );
 		}
 
 		if( NA_IS_OBJECT_ACTION( serializable )){
-			read_done_load_profiles( reader, NA_OBJECT_ACTION( serializable ), ( NadpReaderData * ) reader_data, messages );
-		}
-
-		if( NA_IS_OBJECT_PROFILE( serializable )){
-			read_done_attach_profile( reader, NA_OBJECT_PROFILE( serializable ), ( NadpReaderData * ) reader_data, messages );
+			read_done_action_read_profiles( reader, NA_OBJECT_ACTION( serializable ), ( NadpReaderData * ) reader_data, messages );
 		}
 
+		g_debug( "%s: quitting for %s at %p", thisfn, G_OBJECT_TYPE_NAME( serializable ), ( void * ) serializable );
 	}
 }
 
 static gboolean
-read_done_desktop_is_writable( const NAIFactoryProvider *provider, NAObjectItem *item, NadpReaderData *reader_data, GSList **messages )
+read_done_item_is_writable( const NAIFactoryProvider *provider, NAObjectItem *item, NadpReaderData *reader_data, GSList **messages )
 {
 	NadpDesktopFile *ndf;
-	gchar *path;
+	gchar *uri;
 	gboolean writable;
 
 	ndf = reader_data->ndf;
-	path = nadp_desktop_file_get_key_file_path( ndf );
-	writable = nadp_utils_is_writable_file( path );
-	g_free( path );
+	uri = nadp_desktop_file_get_key_file_uri( ndf );
+	writable = nadp_utils_uri_is_writable( uri );
+	g_free( uri );
 
 	return( writable );
 }
 
-static void
-read_subitems_key( const NAIFactoryProvider *provider, NAObjectItem *item, NadpReaderData *reader_data, GSList **messages )
-{
-	GSList *subitems;
-	gboolean key_found;
-
-	subitems = nadp_desktop_file_get_string_list( reader_data->ndf,
-			NADP_GROUP_DESKTOP,
-			NA_IS_OBJECT_ACTION( item ) ? NADP_KEY_PROFILES : NADP_KEY_ITEMS_LIST,
-			&key_found,
-			NULL );
-
-	if( key_found ){
-		na_object_set_items_slist( item, subitems );
-	}
-
-	na_core_utils_slist_free( subitems );
-}
-
 /*
  * read and attach profiles in the specified order
  * profiles which may exist in .desktop files, but are not referenced
  * in the 'Profiles' string list are just ignored
  */
 static void
-read_done_load_profiles( const NAIFactoryProvider *provider, NAObjectAction *action, NadpReaderData *reader_data, GSList **messages )
+read_done_action_read_profiles( const NAIFactoryProvider *provider, NAObjectAction *action, NadpReaderData *reader_data, GSList **messages )
 {
 	GSList *order;
 	GSList *ip;
@@ -529,6 +648,8 @@ read_done_load_profiles( const NAIFactoryProvider *provider, NAObjectAction *act
 			read_done_action_load_profile( provider, reader_data, profile_id, messages );
 		}
 	}
+
+	na_core_utils_slist_free( order );
 }
 
 static void
@@ -547,9 +668,3 @@ read_done_action_load_profile( const NAIFactoryProvider *provider, NadpReaderDat
 			NA_IFACTORY_OBJECT( profile ),
 			messages );
 }
-
-static void
-read_done_attach_profile( const NAIFactoryProvider *provider, NAObjectProfile *profile, NadpReaderData *reader_data, GSList **messages )
-{
-	na_object_attach_profile( reader_data->action, profile );
-}
