@@ -39,6 +39,7 @@
 #include <api/na-object-api.h>
 
 #include <core/na-iprefs.h>
+#include <core/na-factory-object.h>
 
 #include "base-window.h"
 #include "base-iprefs.h"
@@ -49,8 +50,7 @@
 #include "nact-iactions-list.h"
 #include "nact-main-tab.h"
 #include "nact-icommand-tab.h"
-#include "nact-iconditions-tab.h"
-#include "nact-iadvanced-tab.h"
+#include "nact-ischemes-tab.h"
 
 /* private interface data
  */
@@ -63,6 +63,8 @@ struct NactICommandTabInterfacePrivate {
 #define IPREFS_LEGEND_DIALOG				"icommand-legend-dialog"
 #define IPREFS_COMMAND_CHOOSER				"icommand-command-chooser"
 #define IPREFS_FOLDER_URI					"icommand-folder-uri"
+#define IPREFS_WORKING_DIR_DIALOG			"icommand-working-dir-dialog"
+#define IPREFS_WORKING_DIR_URI				"icommand-working-dir-uri"
 
 /* a data set in the LegendDialog GObject
  */
@@ -79,7 +81,6 @@ static void       interface_base_finalize( NactICommandTabInterface *klass );
 
 static void       on_iactions_list_column_edited( NactICommandTab *instance, NAObject *object, gchar *text, gint column );
 static void       on_tab_updatable_selection_changed( NactICommandTab *instance, gint count_selected );
-static gboolean   tab_set_sensitive( NactICommandTab *instance );
 
 static void       check_for_label( NactICommandTab *instance, GtkEntry *entry, const gchar *label );
 static GtkWidget *get_label_entry( NactICommandTab *instance );
@@ -92,9 +93,12 @@ static void       legend_dialog_show( NactICommandTab *instance );
 static void       legend_dialog_hide( NactICommandTab *instance );
 static void       on_label_changed( GtkEntry *entry, NactICommandTab *instance );
 static void       on_legend_clicked( GtkButton *button, NactICommandTab *instance );
+static gboolean   on_legend_dialog_deleted( GtkWidget *dialog, GdkEvent *event, NactICommandTab *instance );
 static void       on_parameters_changed( GtkEntry *entry, NactICommandTab *instance );
 static void       on_path_browse( GtkButton *button, NactICommandTab *instance );
 static void       on_path_changed( GtkEntry *entry, NactICommandTab *instance );
+static void       on_wdir_browse( GtkButton *button, NactICommandTab *instance );
+static void       on_wdir_changed( GtkEntry *entry, NactICommandTab *instance );
 static gchar     *parse_parameters( NactICommandTab *instance );
 static void       set_label_label( NactICommandTab *instance, const gchar *color );
 static void       update_example_label( NactICommandTab *instance, NAObjectProfile *profile );
@@ -183,11 +187,12 @@ nact_icommand_tab_initial_load_toplevel( NactICommandTab *instance )
 {
 	static const gchar *thisfn = "nact_icommand_tab_initial_load_toplevel";
 
-	g_debug( "%s: instance=%p", thisfn, ( void * ) instance );
 	g_return_if_fail( NACT_IS_ICOMMAND_TAB( instance ));
 
 	if( st_initialized && !st_finalized ){
 
+		g_debug( "%s: instance=%p", thisfn, ( void * ) instance );
+
 		nact_iprefs_migrate_key( BASE_WINDOW( instance ), "iconditions-legend-dialog", IPREFS_LEGEND_DIALOG );
 		nact_iprefs_migrate_key( BASE_WINDOW( instance ), "iconditions-command-chooser", IPREFS_COMMAND_CHOOSER );
 		nact_iprefs_migrate_key( BASE_WINDOW( instance ), "iconditions-folder-uri", IPREFS_FOLDER_URI );
@@ -205,14 +210,16 @@ void
 nact_icommand_tab_runtime_init_toplevel( NactICommandTab *instance )
 {
 	static const gchar *thisfn = "nact_icommand_tab_runtime_init_toplevel";
-	GtkWidget *label_entry, *path_entry, *parameters_entry;
-	GtkButton *path_button, *legend_button;
+	GtkWindow *legend_dialog;
+	GtkWidget *label_entry, *path_entry, *parameters_entry, *wdir_entry;
+	GtkButton *path_button, *legend_button, *wdir_button;
 
-	g_debug( "%s: instance=%p", thisfn, ( void * ) instance );
 	g_return_if_fail( NACT_IS_ICOMMAND_TAB( instance ));
 
 	if( st_initialized && !st_finalized ){
 
+		g_debug( "%s: instance=%p", thisfn, ( void * ) instance );
+
 		label_entry = get_label_entry( instance );
 		base_window_signal_connect(
 				BASE_WINDOW( instance ),
@@ -248,6 +255,27 @@ nact_icommand_tab_runtime_init_toplevel( NactICommandTab *instance )
 				"clicked",
 				G_CALLBACK( on_legend_clicked ));
 
+		legend_dialog = get_legend_dialog( instance );
+		base_window_signal_connect(
+				BASE_WINDOW( instance ),
+				G_OBJECT( legend_dialog ),
+				"delete-event",
+				G_CALLBACK( on_legend_dialog_deleted ));
+
+		wdir_entry = base_window_get_widget( BASE_WINDOW( instance ), "WorkingDirectoryEntry" );
+		base_window_signal_connect(
+				BASE_WINDOW( instance ),
+				G_OBJECT( wdir_entry ),
+				"changed",
+				G_CALLBACK( on_wdir_changed ));
+
+		wdir_button = GTK_BUTTON( base_window_get_widget( BASE_WINDOW( instance ), "CommandWorkingDirectoryButton" ));
+		base_window_signal_connect(
+				BASE_WINDOW( instance ),
+				G_OBJECT( wdir_button ),
+				"clicked",
+				G_CALLBACK( on_wdir_browse ));
+
 		base_window_signal_connect(
 				BASE_WINDOW( instance ),
 				G_OBJECT( instance ),
@@ -267,10 +295,11 @@ nact_icommand_tab_all_widgets_showed( NactICommandTab *instance )
 {
 	static const gchar *thisfn = "nact_icommand_tab_all_widgets_showed";
 
-	g_debug( "%s: instance=%p", thisfn, ( void * ) instance );
 	g_return_if_fail( NACT_IS_ICOMMAND_TAB( instance ));
 
 	if( st_initialized && !st_finalized ){
+
+		g_debug( "%s: instance=%p", thisfn, ( void * ) instance );
 	}
 }
 
@@ -285,10 +314,12 @@ nact_icommand_tab_dispose( NactICommandTab *instance )
 {
 	static const gchar *thisfn = "nact_icommand_tab_dispose";
 
-	g_debug( "%s: instance=%p", thisfn, ( void * ) instance );
 	g_return_if_fail( NACT_IS_ICOMMAND_TAB( instance ));
 
 	if( st_initialized && !st_finalized ){
+
+		g_debug( "%s: instance=%p", thisfn, ( void * ) instance );
+
 		legend_dialog_hide( instance );
 	}
 }
@@ -313,33 +344,34 @@ static void
 on_tab_updatable_selection_changed( NactICommandTab *instance, gint count_selected )
 {
 	static const gchar *thisfn = "nact_icommand_tab_on_tab_updatable_selection_changed";
-	NAObjectItem *item;
 	NAObjectProfile *profile;
-	gboolean enable_tab;
-	GtkWidget *label_entry, *path_entry, *parameters_entry;
-	gchar *label, *path, *parameters;
 	gboolean editable;
+	gboolean enable_tab;
+	GtkWidget *label_entry, *path_entry, *parameters_entry, *wdir_entry;
+	gchar *label, *path, *parameters, *wdir;
 	GtkButton *path_button;
 	GtkButton *legend_button;
 
-	g_debug( "%s: instance=%p, count_selected=%d", thisfn, ( void * ) instance, count_selected );
 	g_return_if_fail( NACT_IS_ICOMMAND_TAB( instance ));
 
 	if( st_initialized && !st_finalized ){
 
-		st_on_selection_change = TRUE;
+		g_debug( "%s: instance=%p, count_selected=%d", thisfn, ( void * ) instance, count_selected );
 
 		g_object_get(
 				G_OBJECT( instance ),
-				TAB_UPDATABLE_PROP_EDITED_ACTION, &item,
-				TAB_UPDATABLE_PROP_EDITED_PROFILE, &profile,
+				TAB_UPDATABLE_PROP_SELECTED_PROFILE, &profile,
 				TAB_UPDATABLE_PROP_EDITABLE, &editable,
 				NULL );
 
-		enable_tab = tab_set_sensitive( instance );
+		enable_tab = ( profile != NULL );
+		nact_main_tab_enable_page( NACT_MAIN_WINDOW( instance ), TAB_COMMAND, enable_tab );
+
+		st_on_selection_change = TRUE;
 
 		label_entry = get_label_entry( instance );
 		label = profile ? na_object_get_label( profile ) : g_strdup( "" );
+		label = label ? label : g_strdup( "" );
 		gtk_entry_set_text( GTK_ENTRY( label_entry ), label );
 		check_for_label( instance, GTK_ENTRY( label_entry ), label );
 		g_free( label );
@@ -348,6 +380,7 @@ on_tab_updatable_selection_changed( NactICommandTab *instance, gint count_select
 
 		path_entry = get_path_entry( instance );
 		path = profile ? na_object_get_path( profile ) : g_strdup( "" );
+		path = path ? path : g_strdup( "" );
 		gtk_entry_set_text( GTK_ENTRY( path_entry ), path );
 		g_free( path );
 		gtk_widget_set_sensitive( path_entry, profile != NULL );
@@ -359,6 +392,7 @@ on_tab_updatable_selection_changed( NactICommandTab *instance, gint count_select
 
 		parameters_entry = get_parameters_entry( instance );
 		parameters = profile ? na_object_get_parameters( profile ) : g_strdup( "" );
+		parameters = parameters ? parameters : g_strdup( "" );
 		gtk_entry_set_text( GTK_ENTRY( parameters_entry ), parameters );
 		g_free( parameters );
 		gtk_widget_set_sensitive( parameters_entry, profile != NULL );
@@ -367,31 +401,22 @@ on_tab_updatable_selection_changed( NactICommandTab *instance, gint count_select
 		legend_button = get_legend_button( instance );
 		gtk_widget_set_sensitive( GTK_WIDGET( legend_button ), profile != NULL );
 
+		wdir_entry = base_window_get_widget( BASE_WINDOW( instance ), "WorkingDirectoryEntry" );
+		wdir = profile ? na_object_get_working_dir( profile ) : g_strdup( "" );
+		wdir = wdir ? wdir : g_strdup( "" );
+		gtk_entry_set_text( GTK_ENTRY( wdir_entry ), wdir );
+		g_free( wdir );
+		gtk_widget_set_sensitive( wdir_entry, profile != NULL );
+		nact_gtk_utils_set_editable( GTK_OBJECT( wdir_entry ), editable );
+
 		st_on_selection_change = FALSE;
 	}
 }
 
-static gboolean
-tab_set_sensitive( NactICommandTab *instance )
-{
-	NAObjectProfile *profile;
-	gboolean enable_tab;
-
-	g_object_get(
-			G_OBJECT( instance ),
-			TAB_UPDATABLE_PROP_EDITED_PROFILE, &profile,
-			NULL );
-
-	enable_tab = ( profile != NULL );
-	nact_main_tab_enable_page( NACT_MAIN_WINDOW( instance ), TAB_COMMAND, enable_tab );
-
-	return( enable_tab );
-}
-
 static void
 check_for_label( NactICommandTab *instance, GtkEntry *entry, const gchar *label )
 {
-	NAObjectProfile *edited;
+	NAObjectProfile *profile;
 
 	nact_main_statusbar_hide_status(
 			NACT_MAIN_WINDOW( instance ),
@@ -401,10 +426,10 @@ check_for_label( NactICommandTab *instance, GtkEntry *entry, const gchar *label
 
 	g_object_get(
 			G_OBJECT( instance ),
-			TAB_UPDATABLE_PROP_EDITED_PROFILE, &edited,
+			TAB_UPDATABLE_PROP_SELECTED_PROFILE, &profile,
 			NULL );
 
-	if( edited && g_utf8_strlen( label, -1 ) == 0 ){
+	if( profile && g_utf8_strlen( label, -1 ) == 0 ){
 
 		/* i18n: status bar message when the profile label is empty */
 		nact_main_statusbar_display_status(
@@ -464,12 +489,12 @@ legend_dialog_hide( NactICommandTab *instance )
 			g_object_get_data( G_OBJECT( legend_dialog ), ICOMMAND_TAB_LEGEND_VISIBLE ));
 
 	if( is_visible ){
-		g_assert( GTK_IS_WINDOW( legend_dialog ));
+		g_return_if_fail( GTK_IS_WINDOW( legend_dialog ));
 		base_iprefs_save_named_window_position( BASE_WINDOW( instance ), legend_dialog, IPREFS_LEGEND_DIALOG );
 		gtk_widget_hide( GTK_WIDGET( legend_dialog ));
 
 		/* set the legend button state consistent for when the dialog is
-		 * hidden by another mean (eg. close the edit profile dialog)
+		 * hidden by another way (eg. close the edit profile dialog)
 		 */
 		legend_button = get_legend_button( instance );
 		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( legend_button ), FALSE );
@@ -499,20 +524,19 @@ legend_dialog_show( NactICommandTab *instance )
 static void
 on_label_changed( GtkEntry *entry, NactICommandTab *instance )
 {
-	NAObjectProfile *edited;
+	NAObjectProfile *profile;
 	const gchar *label;
 
 	if( !st_on_selection_change ){
-
 		g_object_get(
 				G_OBJECT( instance ),
-				TAB_UPDATABLE_PROP_EDITED_PROFILE, &edited,
+				TAB_UPDATABLE_PROP_SELECTED_PROFILE, &profile,
 				NULL );
 
-		if( edited ){
+		if( profile ){
 			label = gtk_entry_get_text( entry );
-			na_object_set_label( edited, label );
-			g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ITEM_UPDATED, edited, TRUE );
+			na_object_set_label( profile, label );
+			g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ITEM_UPDATED, profile, TRUE );
 			check_for_label( instance, entry, label );
 		}
 	}
@@ -529,22 +553,29 @@ on_legend_clicked( GtkButton *button, NactICommandTab *instance )
 	}
 }
 
+static gboolean
+on_legend_dialog_deleted( GtkWidget *dialog, GdkEvent *event, NactICommandTab *instance )
+{
+	/*g_debug( "on_legend_dialog_deleted" );*/
+	legend_dialog_hide( instance );
+	return( TRUE );
+}
+
 static void
 on_parameters_changed( GtkEntry *entry, NactICommandTab *instance )
 {
-	NAObjectProfile *edited;
+	NAObjectProfile *profile;
 
 	if( !st_on_selection_change ){
-
 		g_object_get(
 				G_OBJECT( instance ),
-				TAB_UPDATABLE_PROP_EDITED_PROFILE, &edited,
+				TAB_UPDATABLE_PROP_SELECTED_PROFILE, &profile,
 				NULL );
 
-		if( edited ){
-			na_object_set_parameters( edited, gtk_entry_get_text( entry ));
-			g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ITEM_UPDATED, edited, FALSE );
-			update_example_label( instance, edited );
+		if( profile ){
+			na_object_set_parameters( profile, gtk_entry_get_text( entry ));
+			g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ITEM_UPDATED, profile, FALSE );
+			update_example_label( instance, profile );
 		}
 	}
 }
@@ -552,98 +583,84 @@ on_parameters_changed( GtkEntry *entry, NactICommandTab *instance )
 static void
 on_path_browse( GtkButton *button, NactICommandTab *instance )
 {
-	gboolean set_current_location = FALSE;
-	gchar *uri = NULL;
-	NactApplication *application;
-	NAUpdater *updater;
-	GtkWindow *toplevel;
-	GtkWidget *dialog;
-	GtkWidget *path_entry;
-	const gchar *path;
-	gchar *filename;
-
-	application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( instance )));
-	updater = nact_application_get_updater( application );
-	toplevel = base_window_get_toplevel( BASE_WINDOW( instance ));
-
-	dialog = gtk_file_chooser_dialog_new(
-			_( "Choosing a command" ),
-			toplevel,
-			GTK_FILE_CHOOSER_ACTION_OPEN,
-			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
-			NULL
-			);
-
-	base_iprefs_position_named_window( BASE_WINDOW( instance ), GTK_WINDOW( dialog ), IPREFS_COMMAND_CHOOSER );
-
-	path_entry = get_path_entry( instance );
-	path = gtk_entry_get_text( GTK_ENTRY( path_entry ));
-
-	if( path && strlen( path )){
-		set_current_location = gtk_file_chooser_set_filename( GTK_FILE_CHOOSER( dialog ), path );
-
-	} else {
-		uri = na_iprefs_read_string( NA_IPREFS( updater ), IPREFS_FOLDER_URI, "file:///bin" );
-		gtk_file_chooser_set_current_folder_uri( GTK_FILE_CHOOSER( dialog ), uri );
-		g_free( uri );
-	}
-
-	if( gtk_dialog_run( GTK_DIALOG( dialog )) == GTK_RESPONSE_ACCEPT ){
-		filename = gtk_file_chooser_get_filename( GTK_FILE_CHOOSER( dialog ));
-		gtk_entry_set_text( GTK_ENTRY( path_entry ), filename );
-	    g_free (filename);
-	  }
-
-	uri = gtk_file_chooser_get_current_folder_uri( GTK_FILE_CHOOSER( dialog ));
-	nact_iprefs_write_string( BASE_WINDOW( instance ), IPREFS_FOLDER_URI, uri );
-	g_free( uri );
-
-	base_iprefs_save_named_window_position( BASE_WINDOW( instance ), GTK_WINDOW( dialog ), IPREFS_COMMAND_CHOOSER );
-
-	gtk_widget_destroy( dialog );
+	nact_gtk_utils_select_file(
+			BASE_WINDOW( instance ),
+			_( "Choosing a command" ), IPREFS_COMMAND_CHOOSER,
+			get_path_entry( instance ), IPREFS_FOLDER_URI, "file:///bin" );
 }
 
 static void
 on_path_changed( GtkEntry *entry, NactICommandTab *instance )
 {
-	NAObjectProfile *edited;
+	NAObjectProfile *profile;
 
 	if( !st_on_selection_change ){
-
 		g_object_get(
 				G_OBJECT( instance ),
-				TAB_UPDATABLE_PROP_EDITED_PROFILE, &edited,
+				TAB_UPDATABLE_PROP_SELECTED_PROFILE, &profile,
 				NULL );
 
-		if( edited ){
+		if( profile ){
+			na_object_set_path( profile, gtk_entry_get_text( entry ));
+			g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ITEM_UPDATED, profile, FALSE );
+			update_example_label( instance, profile );
+		}
+	}
+}
 
-			na_object_set_path( edited, gtk_entry_get_text( entry ));
-			g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ITEM_UPDATED, edited, FALSE );
-			update_example_label( instance, edited );
+static void
+on_wdir_browse( GtkButton *button, NactICommandTab *instance )
+{
+	GtkWidget *wdir_entry;
+	NAObjectProfile *profile;
+	gchar *default_value;
+
+	g_object_get(
+			G_OBJECT( instance ),
+			TAB_UPDATABLE_PROP_SELECTED_PROFILE, &profile,
+			NULL );
+
+	if( profile ){
+		default_value = na_factory_object_get_default( NA_IFACTORY_OBJECT( profile ), NAFO_DATA_WORKING_DIR );
+		wdir_entry = base_window_get_widget( BASE_WINDOW( instance ), "WorkingDirectoryEntry" );
+
+		nact_gtk_utils_select_dir(
+				BASE_WINDOW( instance ),
+				_( "Choosing a working directory" ), IPREFS_WORKING_DIR_DIALOG,
+				wdir_entry, IPREFS_WORKING_DIR_URI, default_value );
+
+		g_free( default_value );
+	}
+}
+
+static void
+on_wdir_changed( GtkEntry *entry, NactICommandTab *instance )
+{
+	NAObjectProfile *profile;
+
+	if( !st_on_selection_change ){
+		g_object_get(
+				G_OBJECT( instance ),
+				TAB_UPDATABLE_PROP_SELECTED_PROFILE, &profile,
+				NULL );
+
+		if( profile ){
+			na_object_set_working_dir( profile, gtk_entry_get_text( entry ));
+			g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ITEM_UPDATED, profile, FALSE );
 		}
 	}
 }
 
 /*
- * Valid parameters :
- *
- * %d : base dir of the (first) selected file(s)/folder(s)
- * %f : the name of the (firstà selected file/folder
- * %h : hostname of the (first) URI
- * %m : list of the basename of the selected files/directories separated by space.
- * %M : list of the selected files/directories with their complete path separated by space.
- * %p : port number of the (first) URI
- * %R : space-separated list of selected URIs
- * %s : scheme of the (first) URI
- * %u : (first) URI
- * %U : username of the (first) URI
- * %% : a percent sign
+ * See core/na-tokens.c for valid parameters
  */
 static gchar *
 parse_parameters( NactICommandTab *instance )
 {
 	GString *tmp_string = g_string_new( "" );
+	NAObjectItem *item;
+	NAObjectProfile *profile;
+	NAIContext *context;
 
 	/* i18n notes: example strings for the command preview */
 	gchar *ex_path = _( "/path/to" );
@@ -681,9 +698,21 @@ parse_parameters( NactICommandTab *instance )
 
 	g_string_append_printf( tmp_string, "%s ", command );
 
-	nact_iconditions_tab_get_isfiledir( NACT_ICONDITIONS_TAB( instance ), &is_file, &is_dir );
-	accept_multiple = nact_iconditions_tab_get_multiple( NACT_ICONDITIONS_TAB( instance ));
-	scheme_list = nact_iadvanced_tab_get_schemes( NACT_IADVANCED_TAB( instance ));
+	/*nact_iconditions_tab_get_isfiledir( NACT_ICONDITIONS_TAB( instance ), &is_file, &is_dir );
+	accept_multiple = nact_iconditions_tab_get_multiple( NACT_ICONDITIONS_TAB( instance ));*/
+	is_file = TRUE;
+	is_dir = TRUE;
+	accept_multiple = TRUE;
+
+	g_object_get(
+			G_OBJECT( instance ),
+			TAB_UPDATABLE_PROP_SELECTED_ITEM, &item,
+			TAB_UPDATABLE_PROP_SELECTED_PROFILE, &profile,
+			NULL );
+
+	context = ( profile ? NA_ICONTEXT( profile ) : ( NAIContext * ) item );
+
+	scheme_list = na_object_get_schemes( context );
 
 	separator = g_strdup_printf( " %s/", ex_path );
 	start = g_strdup_printf( "%s/", ex_path );
