@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,12 +32,15 @@
 #include <config.h>
 #endif
 
-#include <glib/gi18n.h>
+#include <api/na-iimporter.h>
 
-#include <common/na-iprefs.h>
+#include <core/na-iprefs.h>
 
-#include "base-iprefs.h"
 #include "nact-application.h"
+#include "nact-iprefs.h"
+#include "nact-export-format.h"
+#include "nact-schemes-list.h"
+#include "nact-providers-list.h"
 #include "nact-preferences-editor.h"
 
 /* private class data
@@ -49,7 +52,8 @@ struct NactPreferencesEditorClassPrivate {
 /* private instance data
  */
 struct NactPreferencesEditorPrivate {
-	gboolean         dispose_has_run;
+	gboolean     dispose_has_run;
+	GConfClient *gconf;
 };
 
 static GObjectClass *st_parent_class = NULL;
@@ -62,13 +66,16 @@ static void     instance_finalize( GObject *dialog );
 
 static NactPreferencesEditor *preferences_editor_new( BaseWindow *parent );
 
-static gchar   *base_get_iprefs_window_id( BaseWindow *window );
-static gchar   *base_get_dialog_name( BaseWindow *window );
+static gchar   *base_get_iprefs_window_id( const BaseWindow *window );
+static gchar   *base_get_dialog_name( const BaseWindow *window );
+static gchar   *base_get_ui_filename( const BaseWindow *dialog );
 static void     on_base_initial_load_dialog( NactPreferencesEditor *editor, gpointer user_data );
 static void     on_base_runtime_init_dialog( NactPreferencesEditor *editor, gpointer user_data );
 static void     on_base_all_widgets_showed( NactPreferencesEditor *editor, gpointer user_data );
+static void     on_esc_quit_toggled( GtkToggleButton *button, NactPreferencesEditor *editor );
 static void     on_cancel_clicked( GtkButton *button, NactPreferencesEditor *editor );
 static void     on_ok_clicked( GtkButton *button, NactPreferencesEditor *editor );
+static void     enable_order_mode_buttons( NactPreferencesEditor *editor );
 static void     save_preferences( NactPreferencesEditor *editor );
 
 static gboolean base_dialog_response( GtkDialog *dialog, gint code, BaseWindow *window );
@@ -131,6 +138,7 @@ class_init( NactPreferencesEditorClass *klass )
 	base_class->dialog_response = base_dialog_response;
 	base_class->get_toplevel_name = base_get_dialog_name;
 	base_class->get_iprefs_window_id = base_get_iprefs_window_id;
+	base_class->get_ui_filename = base_get_ui_filename;
 }
 
 static void
@@ -163,6 +171,8 @@ instance_init( GTypeInstance *instance, gpointer klass )
 			BASE_WINDOW_SIGNAL_ALL_WIDGETS_SHOWED,
 			G_CALLBACK( on_base_all_widgets_showed));
 
+	self->private->gconf = gconf_client_get_default();
+
 	self->private->dispose_has_run = FALSE;
 }
 
@@ -180,6 +190,11 @@ instance_dispose( GObject *dialog )
 
 		self->private->dispose_has_run = TRUE;
 
+		g_object_unref( self->private->gconf );
+
+		nact_schemes_list_dispose( BASE_WINDOW( self ));
+		nact_providers_list_dispose( BASE_WINDOW( self ));
+
 		/* chain up to the parent class */
 		if( G_OBJECT_CLASS( st_parent_class )->dispose ){
 			G_OBJECT_CLASS( st_parent_class )->dispose( dialog );
@@ -239,24 +254,45 @@ nact_preferences_editor_run( BaseWindow *parent )
 }
 
 static gchar *
-base_get_iprefs_window_id( BaseWindow *window )
+base_get_iprefs_window_id( const BaseWindow *window )
 {
 	return( g_strdup( "preferences-editor" ));
 }
 
 static gchar *
-base_get_dialog_name( BaseWindow *window )
+base_get_dialog_name( const BaseWindow *window )
 {
 	return( g_strdup( "PreferencesDialog" ));
 }
 
+static gchar *
+base_get_ui_filename( const BaseWindow *dialog )
+{
+	return( g_strdup( PKGDATADIR "/nact-preferences.ui" ));
+}
+
 static void
 on_base_initial_load_dialog( NactPreferencesEditor *editor, gpointer user_data )
 {
 	static const gchar *thisfn = "nact_preferences_editor_on_initial_load_dialog";
+	NactApplication *application;
+	NAUpdater *updater;
+	GtkWidget *container;
+	GtkTreeView *listview;
 
 	g_debug( "%s: editor=%p, user_data=%p", thisfn, ( void * ) editor, ( void * ) user_data );
 	g_return_if_fail( NACT_IS_PREFERENCES_EDITOR( editor ));
+
+	application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( editor )));
+	updater = nact_application_get_updater( application );
+	container = base_window_get_widget( BASE_WINDOW( editor ), "PreferencesExportFormatVBox" );
+	nact_export_format_init_display( NA_PIVOT( updater ), container, EXPORT_FORMAT_DISPLAY_PREFERENCES );
+
+	listview = GTK_TREE_VIEW( base_window_get_widget( BASE_WINDOW( editor ), "SchemesTreeView" ));
+	nact_schemes_list_create_model( listview, SCHEMES_LIST_FOR_PREFERENCES );
+
+	listview = GTK_TREE_VIEW( base_window_get_widget( BASE_WINDOW( editor ), "ProvidersTreeView" ));
+	nact_providers_list_create_model( listview );
 }
 
 static void
@@ -264,22 +300,26 @@ on_base_runtime_init_dialog( NactPreferencesEditor *editor, gpointer user_data )
 {
 	static const gchar *thisfn = "nact_preferences_editor_on_runtime_init_dialog";
 	NactApplication *application;
-	NAPivot *pivot;
+	NAUpdater *updater;
 	gint order_mode;
 	gboolean add_about_item;
 	gboolean create_root_menu;
 	gboolean relabel;
-	gint import_mode, export_format;
+	guint import_mode;
+	GQuark export_format;
+	GtkWidget *container;
 	GtkWidget *button;
+	gboolean esc_quit, esc_confirm;
+	GtkTreeView *listview;
 
 	g_debug( "%s: editor=%p, user_data=%p", thisfn, ( void * ) editor, ( void * ) user_data );
 
 	application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( editor )));
-	pivot = nact_application_get_pivot( application );
+	updater = nact_application_get_updater( application );
 
 	/* first tab: runtime preferences
 	 */
-	order_mode = na_iprefs_get_order_mode( NA_IPREFS( pivot ));
+	order_mode = na_iprefs_get_order_mode( NA_IPREFS( updater ));
 	switch( order_mode ){
 		case IPREFS_ORDER_ALPHA_ASCENDING:
 			button = base_window_get_widget( BASE_WINDOW( editor ), "OrderAlphaAscButton" );
@@ -295,46 +335,62 @@ on_base_runtime_init_dialog( NactPreferencesEditor *editor, gpointer user_data )
 			break;
 	}
 	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( button ), TRUE );
+	enable_order_mode_buttons( editor );
 
-	create_root_menu = na_iprefs_should_create_root_menu( NA_IPREFS( pivot ));
+	create_root_menu = na_iprefs_read_bool( NA_IPREFS( updater ), IPREFS_CREATE_ROOT_MENU, FALSE );
 	button = base_window_get_widget( BASE_WINDOW( editor ), "CreateRootMenuButton" );
 	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( button ), create_root_menu );
 
-	add_about_item = na_iprefs_should_add_about_item( NA_IPREFS( pivot ));
+	add_about_item = na_iprefs_read_bool( NA_IPREFS( updater ), IPREFS_ADD_ABOUT_ITEM, TRUE );
 	button = base_window_get_widget( BASE_WINDOW( editor ), "AddAboutButton" );
 	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( button ), add_about_item );
 
 	/* second tab: ui preferences
 	 */
-	relabel = na_iprefs_read_bool( NA_IPREFS( pivot ), IPREFS_RELABEL_MENUS, FALSE );
+	relabel = na_iprefs_read_bool( NA_IPREFS( updater ), IPREFS_RELABEL_MENUS, FALSE );
 	button = base_window_get_widget( BASE_WINDOW( editor ), "RelabelMenuButton" );
 	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( button ), relabel );
 
-	relabel = na_iprefs_read_bool( NA_IPREFS( pivot ), IPREFS_RELABEL_ACTIONS, FALSE );
+	relabel = na_iprefs_read_bool( NA_IPREFS( updater ), IPREFS_RELABEL_ACTIONS, FALSE );
 	button = base_window_get_widget( BASE_WINDOW( editor ), "RelabelActionButton" );
 	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( button ), relabel );
 
-	relabel = na_iprefs_read_bool( NA_IPREFS( pivot ), IPREFS_RELABEL_PROFILES, FALSE );
+	relabel = na_iprefs_read_bool( NA_IPREFS( updater ), IPREFS_RELABEL_PROFILES, FALSE );
 	button = base_window_get_widget( BASE_WINDOW( editor ), "RelabelProfileButton" );
 	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( button ), relabel );
 
+	button = base_window_get_widget( BASE_WINDOW( editor ), "EscCloseButton" );
+
+	base_window_signal_connect(
+			BASE_WINDOW( editor ),
+			G_OBJECT( button ),
+			"toggled",
+			G_CALLBACK( on_esc_quit_toggled ));
+
+	esc_quit = na_iprefs_read_bool( NA_IPREFS( updater ), IPREFS_ASSIST_ESC_QUIT, TRUE );
+	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( button ), esc_quit );
+
+	esc_confirm = na_iprefs_read_bool( NA_IPREFS( updater ), IPREFS_ASSIST_ESC_CONFIRM, TRUE );
+	button = base_window_get_widget( BASE_WINDOW( editor ), "EscConfirmButton" );
+	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( button ), esc_confirm );
+
 	/* third tab: import tool
 	 */
-	import_mode = na_iprefs_get_import_mode( NA_IPREFS( pivot ), IPREFS_IMPORT_ACTIONS_IMPORT_MODE );
+	import_mode = na_iprefs_get_import_mode( editor->private->gconf, IPREFS_IMPORT_ITEMS_IMPORT_MODE );
 	switch( import_mode ){
-		case IPREFS_IMPORT_ASK:
+		case IMPORTER_MODE_ASK:
 			button = base_window_get_widget( BASE_WINDOW( editor ), "PrefsAskButton" );
 			break;
 
-		case IPREFS_IMPORT_RENUMBER:
+		case IMPORTER_MODE_RENUMBER:
 			button = base_window_get_widget( BASE_WINDOW( editor ), "PrefsRenumberButton" );
 			break;
 
-		case IPREFS_IMPORT_OVERRIDE:
+		case IMPORTER_MODE_OVERRIDE:
 			button = base_window_get_widget( BASE_WINDOW( editor ), "PrefsOverrideButton" );
 			break;
 
-		case IPREFS_IMPORT_NO_IMPORT:
+		case IMPORTER_MODE_NO_IMPORT:
 		default:
 			button = base_window_get_widget( BASE_WINDOW( editor ), "PrefsNoImportButton" );
 			break;
@@ -343,27 +399,22 @@ on_base_runtime_init_dialog( NactPreferencesEditor *editor, gpointer user_data )
 
 	/* fourth tab: export tool
 	 */
-	export_format = na_iprefs_get_export_format( NA_IPREFS( pivot ), IPREFS_EXPORT_FORMAT );
-	switch( export_format ){
-		case IPREFS_EXPORT_FORMAT_ASK:
-			button = base_window_get_widget( BASE_WINDOW( editor ), "PrefsExportAskButton" );
-			break;
+	export_format = nact_iprefs_get_export_format( BASE_WINDOW( editor ), IPREFS_EXPORT_FORMAT );
+	container = base_window_get_widget( BASE_WINDOW( editor ), "PreferencesExportFormatVBox" );
+	nact_export_format_select( container, export_format );
 
-		case IPREFS_EXPORT_FORMAT_GCONF_SCHEMA_V2:
-			button = base_window_get_widget( BASE_WINDOW( editor ), "PrefsExportGConfSchemaV2Button" );
-			break;
+	/* fifth tab: default schemes
+	 */
+	listview = GTK_TREE_VIEW( base_window_get_widget( BASE_WINDOW( editor ), "SchemesTreeView" ));
+	nact_schemes_list_init_view( listview, BASE_WINDOW( editor ), NULL, NULL );
 
-		case IPREFS_EXPORT_FORMAT_GCONF_SCHEMA_V1:
-			button = base_window_get_widget( BASE_WINDOW( editor ), "PrefsExportGConfSchemaV1Button" );
-			break;
-
-		case IPREFS_EXPORT_FORMAT_GCONF_ENTRY:
-		default:
-			button = base_window_get_widget( BASE_WINDOW( editor ), "PrefsExportGConfDumpButton" );
-			break;
-	}
-	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( button ), TRUE );
+	/* sixth tab: I/O providers priorities
+	 */
+	listview = GTK_TREE_VIEW( base_window_get_widget( BASE_WINDOW( editor ), "ProvidersTreeView" ));
+	nact_providers_list_init_view( listview, BASE_WINDOW( editor ));
 
+	/* dialog buttons
+	 */
 	base_window_signal_connect_by_name(
 			BASE_WINDOW( editor ),
 			"CancelButton",
@@ -386,6 +437,19 @@ on_base_all_widgets_showed( NactPreferencesEditor *editor, gpointer user_data )
 	g_debug( "%s: editor=%p, user_data=%p", thisfn, ( void * ) editor, ( void * ) user_data );
 	notebook = GTK_NOTEBOOK( base_window_get_widget( BASE_WINDOW( editor ), "PreferencesNotebook" ));
 	gtk_notebook_set_current_page( notebook, 0 );
+
+	nact_schemes_list_show_all( BASE_WINDOW( editor ));
+}
+
+static void
+on_esc_quit_toggled( GtkToggleButton *button, NactPreferencesEditor *editor )
+{
+	gboolean is_active;
+	GtkWidget *toggle;
+
+	is_active = gtk_toggle_button_get_active( button );
+	toggle = base_window_get_widget( BASE_WINDOW( editor ), "EscConfirmButton" );
+	gtk_widget_set_sensitive( toggle, is_active );
 }
 
 static void
@@ -404,19 +468,44 @@ on_ok_clicked( GtkButton *button, NactPreferencesEditor *editor )
 	gtk_dialog_response( GTK_DIALOG( toplevel ), GTK_RESPONSE_OK );
 }
 
+static void
+enable_order_mode_buttons( NactPreferencesEditor *editor )
+{
+	NactApplication *application;
+	NAUpdater *updater;
+	gboolean writable;
+	GtkWidget *button;
+
+	application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( editor )));
+	updater = nact_application_get_updater( application );
+	writable = na_pivot_is_level_zero_writable( NA_PIVOT( updater ));
+
+	button = base_window_get_widget( BASE_WINDOW( editor ), "OrderAlphaAscButton" );
+	gtk_widget_set_sensitive( button, writable );
+
+	button = base_window_get_widget( BASE_WINDOW( editor ), "OrderAlphaDescButton" );
+	gtk_widget_set_sensitive( button, writable );
+
+	button = base_window_get_widget( BASE_WINDOW( editor ), "OrderManualButton" );
+	gtk_widget_set_sensitive( button, writable );
+}
+
 static void
 save_preferences( NactPreferencesEditor *editor )
 {
 	NactApplication *application;
-	NAPivot *pivot;
+	NAUpdater *updater;
 	GtkWidget *button;
 	gint order_mode;
 	gboolean enabled;
 	gboolean relabel;
-	gint import_mode, export_format;
+	guint import_mode;
+	GtkWidget *container;
+	NAExportFormat *export_format;
+	gboolean esc_quit, esc_confirm;
 
 	application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( editor )));
-	pivot = nact_application_get_pivot( application );
+	updater = nact_application_get_updater( application );
 
 	/* first tab: runtime preferences
 	 */
@@ -435,67 +524,71 @@ save_preferences( NactPreferencesEditor *editor )
 			}
 		}
 	}
-	na_iprefs_set_order_mode( NA_IPREFS( pivot ), order_mode );
+	na_iprefs_set_order_mode( NA_IPREFS( updater ), order_mode );
 
 	button = base_window_get_widget( BASE_WINDOW( editor ), "CreateRootMenuButton" );
 	enabled = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( button ));
-	na_iprefs_set_create_root_menu( NA_IPREFS( pivot ), enabled );
+	nact_iprefs_write_bool( BASE_WINDOW( editor ), IPREFS_CREATE_ROOT_MENU, enabled );
 
 	button = base_window_get_widget( BASE_WINDOW( editor ), "AddAboutButton" );
 	enabled = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( button ));
-	na_iprefs_set_add_about_item( NA_IPREFS( pivot ), enabled );
+	nact_iprefs_write_bool( BASE_WINDOW( editor ), IPREFS_ADD_ABOUT_ITEM, enabled );
 
 	/* second tab: runtime preferences
 	 */
 	button = base_window_get_widget( BASE_WINDOW( editor ), "RelabelMenuButton" );
 	relabel = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( button ));
-	na_iprefs_write_bool( NA_IPREFS( pivot ), IPREFS_RELABEL_MENUS, relabel );
+	nact_iprefs_write_bool( BASE_WINDOW( editor ), IPREFS_RELABEL_MENUS, relabel );
 
 	button = base_window_get_widget( BASE_WINDOW( editor ), "RelabelActionButton" );
 	relabel = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( button ));
-	na_iprefs_write_bool( NA_IPREFS( pivot ), IPREFS_RELABEL_ACTIONS, relabel );
+	nact_iprefs_write_bool( BASE_WINDOW( editor ), IPREFS_RELABEL_ACTIONS, relabel );
 
 	button = base_window_get_widget( BASE_WINDOW( editor ), "RelabelProfileButton" );
 	relabel = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( button ));
-	na_iprefs_write_bool( NA_IPREFS( pivot ), IPREFS_RELABEL_PROFILES, relabel );
+	nact_iprefs_write_bool( BASE_WINDOW( editor ), IPREFS_RELABEL_PROFILES, relabel );
+
+	button = base_window_get_widget( BASE_WINDOW( editor ), "EscCloseButton" );
+	esc_quit = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( button ));
+	nact_iprefs_write_bool( BASE_WINDOW( editor ), IPREFS_ASSIST_ESC_QUIT, esc_quit );
+
+	button = base_window_get_widget( BASE_WINDOW( editor ), "EscConfirmButton" );
+	esc_confirm = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( button ));
+	nact_iprefs_write_bool( BASE_WINDOW( editor ), IPREFS_ASSIST_ESC_CONFIRM, esc_confirm );
 
 	/* third tab: import tool
 	 */
-	import_mode = IPREFS_IMPORT_NO_IMPORT;
+	import_mode = IMPORTER_MODE_NO_IMPORT;
 	button = base_window_get_widget( BASE_WINDOW( editor ), "PrefsRenumberButton" );
 	if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( button ))){
-		import_mode = IPREFS_IMPORT_RENUMBER;
+		import_mode = IMPORTER_MODE_RENUMBER;
 	} else {
 		button = base_window_get_widget( BASE_WINDOW( editor ), "PrefsOverrideButton" );
 		if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( button ))){
-			import_mode = IPREFS_IMPORT_OVERRIDE;
+			import_mode = IMPORTER_MODE_OVERRIDE;
 		} else {
 			button = base_window_get_widget( BASE_WINDOW( editor ), "PrefsAskButton" );
 			if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( button ))){
-				import_mode = IPREFS_IMPORT_ASK;
+				import_mode = IMPORTER_MODE_ASK;
 			}
 		}
 	}
-	na_iprefs_set_import_mode( NA_IPREFS( pivot ), IPREFS_IMPORT_ACTIONS_IMPORT_MODE, import_mode );
+	na_iprefs_set_import_mode( editor->private->gconf, IPREFS_IMPORT_ITEMS_IMPORT_MODE, import_mode );
 
 	/* fourth tab: export tool
 	 */
-	export_format = IPREFS_EXPORT_FORMAT_GCONF_ENTRY;
-	button = base_window_get_widget( BASE_WINDOW( editor ), "PrefsExportGConfSchemaV1Button" );
-	if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( button ))){
-		export_format = IPREFS_EXPORT_FORMAT_GCONF_SCHEMA_V1;
-	} else {
-		button = base_window_get_widget( BASE_WINDOW( editor ), "PrefsExportGConfSchemaV2Button" );
-		if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( button ))){
-			export_format = IPREFS_EXPORT_FORMAT_GCONF_SCHEMA_V2;
-		} else {
-			button = base_window_get_widget( BASE_WINDOW( editor ), "PrefsExportAskButton" );
-			if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( button ))){
-				export_format = IPREFS_EXPORT_FORMAT_ASK;
-			}
-		}
-	}
-	na_iprefs_set_export_format( NA_IPREFS( pivot ), IPREFS_EXPORT_FORMAT, export_format );
+	container = base_window_get_widget( BASE_WINDOW( editor ), "PreferencesExportFormatVBox" );
+	export_format = nact_export_format_get_selected( container );
+	nact_iprefs_set_export_format(
+			BASE_WINDOW( editor ), IPREFS_EXPORT_FORMAT, na_export_format_get_quark( export_format ));
+
+	/* fifth tab: list of default schemes
+	 */
+	nact_schemes_list_save_defaults( BASE_WINDOW( editor ));
+
+	/* sixth tab: priorities of I/O providers
+	 */
+	nact_providers_list_save( BASE_WINDOW( editor ));
 }
 
 static gboolean
